package zero1hd.rhythmbullet.audio;

import java.util.Observable;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.Sort;

import zero1hd.rhythmbullet.util.FileHandleAlphabeticalComparator;

public class MusicList extends Observable {
	private Array<FileHandle> musicList;
	private String searchPath;
	private boolean searched;
	private FileHandleAlphabeticalComparator fhac;
	public MusicList() {
		musicList = new Array<>();
		fhac = new FileHandleAlphabeticalComparator();
	}
	
	public void refresh() {
		searched = true;
		musicList.clear();
		Gdx.app.debug("SongController", "Searching path: " + searchPath);
		if (Gdx.files.absolute(searchPath).exists() && Gdx.files.absolute(searchPath).isDirectory()) {
			musicList.addAll(Gdx.files.absolute(searchPath).list((dir, name) -> {
				if (name.endsWith("mp3") || name.endsWith("wav")) {
					return true;
				}
				return false;
			}));
		}
		if (!Gdx.files.external("RhythmBullet/Alan Walker - Spectre.mp3").exists()) {
			Gdx.files.internal("music/Alan Walker - Spectre.mp3").copyTo(Gdx.files.external("RhythmBullet/Alan Walker - Spectre.mp3"));
		}
		musicList.add(Gdx.files.external("RhythmBullet/Alan Walker - Spectre.mp3"));
		
		Sort.instance().sort(musicList, fhac);
		notifyObservers();
	}
	
	public void setSearchPath(String searchPath) {
		this.searchPath = searchPath;
		setChanged();
	}
	
	public MusicManager getAudioData(FileHandle file) {
		if (file.extension().equalsIgnoreCase("wav")) {
			return new WAVManager(file);
		} else if (file.extension().equalsIgnoreCase("mp3")) {
			return new Mp3Manager(file);
		}
		return null;
	}
	
	public MusicManager getMusicInfoFromIndex(int index) {
		if (!searched) Gdx.app.debug("SongList", "Warning, this list hasn't even searched yet...");
		return getAudioData(musicList.get(index));
	}
	
	public int getAmountOfMusic() {
		return musicList.size;
	}
	
	public FileHandle getSongFileHandleFromIndex(int index) {
		if (!searched) Gdx.app.debug("SongList", "Warning, this list hasn't even searched yet...");
		return musicList.get(index);
	}
	
	public Array<FileHandle> getMusicList() {
		return musicList;
	}
	
	public boolean isSearched() {
		return searched;
	}
}
