package zero1hd.rhythmbullet.audio;

import java.security.InvalidParameterException;
import java.util.Observable;
import java.util.Observer;
import java.util.Random;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.audio.Music.OnCompletionListener;

public class MusicListController extends Observable implements OnCompletionListener, Observer {
	private MusicList musicList;
	private MusicManager mm;
	private int currentPlaybackID;
	private boolean autoPlay;
	private boolean shuffle;
	private Random rand;
	
	private Preferences prefs;
	public MusicListController(MusicList musicList, Preferences prefs) {
		if (prefs == null) throw new NullPointerException("preferences can't be null...");
		if (musicList == null) throw new NullPointerException("music list can't be null...");
		if (!musicList.isSearched()) throw new InvalidParameterException("music list has to be searched already.");
		musicList.addObserver(this);
		this.prefs = prefs;
		this.musicList = musicList;
		rand = new Random();
		changeMusic();
	}
	
	public void play() {
		mm.play();
		mm.setVolume(prefs.getFloat("music vol", 1f));
	}
	
	public void setMusicByIndex(int index) {
		this.currentPlaybackID = index;
		changeMusic();
	}
	
	public void skip() {
		currentPlaybackID++;
		if (shuffle) {
			shuffle(false);
		}
		changeMusic();
	}
	
	public void previous() {
		currentPlaybackID--;
		if (shuffle) {
			shuffle(false);
		}
		changeMusic();
	}

	@Override
	public void onCompletion(Music music) {
		if (autoPlay) {
			if (shuffle) {
				if (musicList.getAmountOfMusic() != 0) {
					currentPlaybackID = rand.nextInt(musicList.getAmountOfMusic());
				} else {
					currentPlaybackID = 0;
				}
			} else {
				currentPlaybackID++;
			}
			changeMusic();
			play();
		}
	}
	
	/**
	 * Shuffles the controller whether the shuffle boolean is true or false.
	 * @param load whether this method should also make sure to load the music, dispose of the last one, etc. Normally called unless you plan to manually call it elswhere.
	 */
	public void shuffle(boolean load) {
		if (musicList.getAmountOfMusic() == 0) {
			currentPlaybackID = 0;
		} else {
			currentPlaybackID = rand.nextInt(musicList.getAmountOfMusic());
		}
		if (load) {
			changeMusic();
		}
	}
	
	public void setAutoPlay(boolean autoPlay) {
		this.autoPlay = autoPlay;
	}
	
	public void setShuffle(boolean shuffle) {
		this.shuffle = shuffle;
	}
	
	public boolean isShuffle() {
		return shuffle;
	}
	
	public boolean isAutoPlay() {
		return autoPlay;
	}
	
	private void changeMusic() {
		if (mm != null) {
			mm.dispose();
		}
		if (currentPlaybackID < 0) {
			currentPlaybackID = musicList.getAmountOfMusic();
		}
		if (currentPlaybackID > musicList.getAmountOfMusic()) {
			currentPlaybackID = 0;
		}
		this.mm = musicList.getMusicInfoFromIndex(currentPlaybackID);
		mm.setOnCompletionListener(this);
		
		setChanged();
		notifyObservers(mm);
	}
	
	public MusicList getMusicList() {
		return musicList;
	}
	
	
	public MusicManager getCurrentMusicManager() {
		return mm;
	}

	@Override
	public void update(Observable o, Object arg) {
		if (o == musicList) {
			changeMusic();
			play();
		}
	}
}
